# This file was auto-generated by zetup
#
# https://bitbucket.org/userzimmermann/zetup.py


from __future__ import absolute_import

from collections import OrderedDict

from zetup.version import Version
from zetup.dist import Distribution
from zetup.requires import Requirements
from zetup.extras import Extras
from zetup.notebook import Notebook


LICENSE = 'LGPLv3'

TITLE = 'zetup'

PYTHON = ['2.7', '3.3', '3.4']

MODULES = []

KEYWORDS = ['setup', 'python3']

PACKAGES = ['zetup', 'zetup.commands', 'zetup.notebook', 'zetup.commands.make']

EMAIL = 'zimmermann.code@gmail.com'

AUTHOR = 'Stefan Zimmermann'

REQUIRES = Requirements("""
setuptools
hgdistver >= 0.23

""")

URL = 'https://bitbucket.org/userzimmermann/zetup.py'

SETUP_HOOKS = []

VERSION = Version('0.2.9.post0')

NAME = 'zetup'

DESCRIPTION = "Zimmermann's Python package setup"

SETUP_REQUIRES = None

EXTRAS = Extras([
('commands', """
path.py #import path
jinjatools >= 0.1.4

"""),
('conda', """
conda

"""),
('pytest', """
pytest

""")
])

DISTRIBUTION = Distribution('zetup', 'zetup', '0.2.9.post0')

CLASSIFIERS = ['Development Status :: 3 - Alpha', 'Intended Audience :: Developers', 'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)', 'Operating System :: OS Independent', 'Topic :: Software Development', 'Topic :: Utilities', 'Programming Language :: Python', 'Programming Language :: Python :: 2.7', 'Programming Language :: Python :: 3.3', 'Programming Language :: Python :: 3.4']